# Generated by roxygen2: do not edit by hand

S3method(plot,std_glm)
S3method(plot,std_surv)
S3method(print,std_custom)
S3method(print,std_glm)
S3method(print,std_surv)
S3method(print,summary.parfrailty)
S3method(summary,parfrailty)
S3method(tidy,std_custom)
S3method(tidy,std_glm)
S3method(tidy,std_surv)
export(parfrailty)
export(plot.std_glm)
export(plot.std_surv)
export(print.std_custom)
export(print.std_glm)
export(print.std_surv)
export(sandwich)
export(standardize)
export(standardize_coxph)
export(standardize_gee)
export(standardize_glm)
export(standardize_glm_dr)
export(standardize_level)
export(standardize_parfrailty)
export(summary.parfrailty)
export(tidy)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,rbindlist)
importFrom(drgee,gee)
importFrom(generics,tidy)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,points)
importFrom(stats,.getXlevels)
importFrom(stats,ave)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,contrasts)
importFrom(stats,cov)
importFrom(stats,drop.terms)
importFrom(stats,family)
importFrom(stats,glm)
importFrom(stats,glm.control)
importFrom(stats,is.empty.model)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,naresid)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,predict.glm)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,stepfun)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weights)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,coxph.detail)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
