# handle various imports here
#' @importFrom data.table data.table .N rbindlist .SD
#' @importFrom drgee gee
#' @importFrom graphics lines mtext points
#' @importFrom survival coxph.detail coxph Surv
#' @importFrom stats ave binomial family glm model.extract model.frame cov drop.terms reformulate update .getXlevels contrasts glm.control is.empty.model model.response
#' @importFrom stats coef model.matrix naresid optim pnorm predict
#' @importFrom stats predict.glm printCoefmat qnorm residuals stepfun terms
#' @importFrom stats var vcov weights
#' @importFrom utils setTxtProgressBar txtProgressBar
#' @docType package
"_PACKAGE"
